#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
	Script python exemple destiné à récupérer les données d'un Arduino
	 et à les servir sur le web
"""

import sys
import threading
import datetime
import serial
import string
import json
#import urllib2
import bottle 


######################################################################

class arduinoData(object):
	"""
		Dernières informations reçues de l'Arduino
		L'exemple fourni les informations "AAA", "BBB" & "CCC"
		Une nouvelle donnée "DDD" est générée et est la moyenne des 4 dernières valeurs de "AAA"
	"""

	def __init__(self):
		self._raw = ""   # données brutes reçues
		self._AAA = 0    # valeur AAA fournie par l'Arduino
		self._BBB = 0    # valeur BBB fournie par l'Arduino
		self._CCC = 0    # valeur CCC fournie par l'Arduino
		self._DDD = 0    # moyenne des 4 dernières valeur de AAA
		self.__A4 = [-1,-1,-1,-1]
		self.__ix = 0
		pass

	def _get_raw(self):
		return(self._raw)
	def _set_raw(self, newRaw):
		self.__setRaw(newRaw)
	raw = property(	fget = _get_raw, 
						fset = _set_raw, 
						doc = "raw, texte brute reçu de l'arduino." )

	def __setRaw(self, newRaw):
		try:
			self._raw = newRaw
			j = json.loads(newRaw)
			self._set_AAA(j['AAA'])
			self._set_BBB(j['BBB'])
			self._set_CCC(j['CCC'])
			try:
				self.__A4[self.__ix] = self._AAA
				self.__ix += 1
				if(self.__ix > 3):
					self.__ix = 0
				m = 0
				n = 0
				for i in range(0,3):
					if(self.__A4[i] >= 0):
						m += self.__A4[i]
						n += 1
				self._DDD = m / n
			except:
				print('DDD error')
		except:
			print("__setraw error")

	def _get_AAA(self):
		return(self._AAA)
	def _set_AAA(self, newAAA):
		self._AAA = newAAA
	AAA = property(
						fget = _get_AAA, 
						fset = _set_AAA, 
						doc = "Valeur de AAA fournie par l'Arduino" )

	def _get_BBB(self):
		return(self._BBB)
	def _set_BBB(self, newBBB):
		self._BBB = newBBB
	BBB = property(	fget = _get_BBB, 
						fset = _set_BBB, 
						doc = "Valeur de BBB fournie par l'Arduino" )

	def _get_CCC(self):
		return(self._CCC)
	def _set_CCC(self, newCCC):
		self._CCC = newCCC
	CCC = property(	fget = _get_CCC, 
						fset = _set_CCC, 
						doc = "Valeur de CCC fournie par l'Arduino" )

	def _get_DDD(self):
		return(self._DDD)
	def _set_DDD(self, newDDD):
		self._DDD = newDDD
	DDD = property(	fget = _get_DDD, 
						fset = _set_DDD, 
						doc = "Moyenne des 4 derniers AAA fournis par l'Arduino" )


######################################################################

class readFromSerial(threading.Thread):
	"""
		Lecture du port USB sur lequel est branché l'Arduino.
		La lecture se fait dans un thread différent.
	"""

	def __init__(self, ad): 
		self._OK = 0
		try:
			threading.Thread.__init__(self)
			self.setDaemon(True)
			self._ad = ad
			print('Initialize serial')
			self._ser = serial.Serial(	port='/dev/ttyUSB0', 
										baudrate=115200, 
										parity=serial.PARITY_EVEN, 
										stopbits=serial.STOPBITS_ONE, 
										bytesize=serial.SEVENBITS,
										timeout=1)
			print('Serial initialized')
			self._OK = 1
		except:
			print('serial arduinoData error')

	def __del__(self):
		self._ser.close()

	def run(self):
		while True:
			try:
				adLine = self._ser.readline().decode("utf-8").strip()
				if (len(adLine) > 0):
					print(adLine)
					if (adLine[0] == '{'):
						self._ad.raw = adLine
						print(self._ad.DDD)
						logFile = open('log.txt', 'a')
						logFile.write("{0} => {1}\n".format(datetime.datetime.now(), adLine))
						logFile.close()
			except:
				#e = sys.exc_info()[0]
				#print( "<p>Error: %s</p>" % e )
				print('Serial no data')
				continue

	def sendCommand(self, cmd):
		print(cmd.encode("ascii"))
		self._ser.write((cmd+"\n").encode("ascii"))


######################################################################

#class sendToServer(threading.Thread):
#	"""
#		Envoie toutes les minutes au server les donnes de l'Arduino.
#		Le travaille se fait dans un thread différent.
#	"""
#
#	def __init__(self, ad): 
#		self._OK = 0
#		try:
#			threading.Thread.__init__(self)
#			self.setDaemon(True)
#			self._ad = ad
#			print 'Initialize server'
#			self._OK = 1
#		except:
#			print 'server KO'
#
#	def run(self):
#		# Il faut utiliser un proxy pour sortir sur Internet
#		proxy_info = { 'host' : 'prxy-xxxxxxx-xxxxxx.yy', 'port' : zzzz }
#		proxy_support = urllib2.ProxyHandler({"http" : "http://%(host)s:%(port)d" % proxy_info})
#		# On créé un opener utilisant ce handler:
#		opener = urllib2.build_opener(proxy_support)
#		# Puis on installe cet opener comme opener par défaut du module urllib2.
#		urllib2.install_opener(opener)
#		baseUrl = "http://Url.Du.Serveur.Internet/chargé/du/datalogging/"
#		while True:
#			try:
#				# prepare l'URL
#				if (ad.AAA >= 0):
#					dataUrl = '{0}idxA={1}&idxB={2}&idxC={3}&idxD={4}'.format(baseUrl, ad.AAA, ad.BBB, ad.CCC, ad.DDD)
#					# envoi au server
#					urllib2.urlopen(dataUrl)
#			except:
#				print 'Sender error'
#			# dort une minute
#			time.sleep(60)


######################################################################

# Instancie un site web
webApp = bottle.Bottle()

@webApp.route('/')
def homePage():
	return bottle.static_file('index.html', root='static')

@webApp.route('/favicon.ico', method='ANY')
def get_favicon():
	return bottle.static_file('favicon.ico', root='static')

@webApp.route('/dataArduino.html', method='ANY')
def dataArduino():
	print(bottle.request.remote_addr)
	return '<head><meta http-equiv="refresh" content="1"></head>' + ad.raw + '</br>' + ('%.2f' % ad.DDD) + '</br><a href="/">retour</a>'

@webApp.route('/commandes.html', method='ANY')
def commandes():
	return bottle.static_file('commandes.html', root='static')

@webApp.route('/static/<fileName>')
def staticPages(fileName):
	return bottle.static_file(fileName, root='static')

@webApp.route('/json', method='ANY')
def jsonData():
	dict = {'AAA':ad.AAA, 'BBB':ad.BBB, 'CCC':ad.CCC, 'DDD':ad.DDD }
	return dict

@webApp.route('/cmd/<cmd>', method='ANY')
def cmd(cmd):
	"""	Rceptionne une commande  envoyer  l'arduino
	"""
	adReader.sendCommand(cmd)
	bottle.redirect('/dataArduino.html')

######################################################################

if __name__ == '__main__':

	# Instancie la classe de gestion des données reçues de l'Arduino
	ad = arduinoData()

	# Instancie & démarre les threads de lecture de l'Arduino
	adReader = readFromSerial(ad)
	adReader.start()

	# Instancie & démarre les threads d'envoie des données
	#adSender = sendToServer(ad)
	#adSender.start()

	# Lance le site web
	bottle.run(webApp, host='0.0.0.0', port=8080, server='cherrypy')
